// TICGI Version 0.6, July 30th 1999. Copyright (c) Tilman Budde 1999

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define TEXTHTML "text/html"
#define TEXTPLAIN "text/plain"

// The main structure for parsing the returned data
typedef struct {
					  char *field;
                 char *value;
					} CGI_dataStruct;

CGI_dataStruct *_CGI_formData;

// Some CGI environment variables you might want to access
char *requestMethod = NULL;
char *remoteAddr;
char *serverSoftware;
char *pathInfo;
char *pathTranslated;
int contentLength = 0;

void setMime (char *ContentType)
// Tells the client the MIME type of the data he's going to return
{
  fprintf (stdout, "Content-Type:%s\n\n", ContentType);
}

char *getRequest (int entries)
// returns the contents of the get or post request the user made
// Note : You are responsible for freeing the returned value
{
  char *query = NULL;
  int i;

  if ((requestMethod = getenv ("REQUEST_METHOD")) == NULL)
  {
    fprintf (stdout, "Error : REQUEST_METHOD is empty");
    return NULL;
  }
  if (strcmp (requestMethod, "GET") == 0)
  // Client requested data via GET
  {
    query = strdup (getenv ("QUERY_STRING"));
    // We don't really need it, but someone might want to know
    contentLength = strlen (query);
  }
  else if (strcmp (requestMethod, "POST") == 0)
  // Client requested data via POST
  {
    contentLength = atoi (getenv ("CONTENT_LENGTH"));
    query = (char *) malloc (contentLength);
    fscanf (stdin, "%s\0", query);
  }

  if ((_CGI_formData = (CGI_dataStruct *) malloc (entries * sizeof (CGI_dataStruct) + 1)) == NULL)
  // Pretty much the worst that can happen
  {
    fprintf (stdout, "Error : Insufficient memory for _CGI_formData or no form fields !");
    return NULL;
  }
  _CGI_formData->field = query;
  _CGI_formData->value = strchr (query, '=') + 1;
  // This is going to work as the sentinel
  _CGI_formData [entries].field = NULL;
  _CGI_formData [entries].value = NULL;

  for (i = 1; i < entries; i++)
  {
    _CGI_formData [i].field = strchr (_CGI_formData [i - 1].value, '&') + 1;
    _CGI_formData [i].value = strchr (_CGI_formData [i].field, '=') + 1;
  }

  // Clearing all entries that were empty
  for (i = 0; i < entries; i++)
  {
    if ((_CGI_formData [i].value [0] == '&') | (_CGI_formData [i].value [0] == '\0'))
      _CGI_formData [i].value = NULL;
  }

  // Getting some leftover CGI variables
  remoteAddr     = getenv ("REMOTE_ADDR");
  serverSoftware = getenv ("SERVER_SOFTWARE");
  pathInfo       = getenv ("PATH_INFO");
  pathTranslated = getenv ("PATH_TRANSLATED");

  return query;
}

char *extractValue (char *field)
// Extracts the value of a field from a CGI string
{
  int i = 0;
  char *tok;

  while (_CGI_formData [i].field != NULL)
  {
    tok = strtok (_CGI_formData [i].field, "=");
    if (strcmp (tok, field) == 0)
      return strtok (_CGI_formData [i].value, "&");
    i++;
  }
  // If he got here, "field" does not exist
  return NULL;
}

int getDecimal (char hex)
// Converts a hexadecimal character to its decimal counterpart
{
  switch (hex)
  {
    case 'F' : return 15;
    case 'E' : return 14;
    case 'D' : return 13;
    case 'C' : return 12;
    case 'B' : return 11;
    case 'A' : return 10;
    default  : return (hex - 48);
  }
}

char *decodeString (char *value)
{
  int i = 0, decValue;

  while (value [i] != '\0')
  {
    if (value [i] == '+') value [i] = ' ';
    else if (value [i] == '%')
    {
      decValue = (getDecimal (value [i + 1]) << 4) + getDecimal (value [i + 2]);
      value [i] = (char) decValue;
      memcpy (value + i + 1, value + i + 3, strlen (value) - i);
    }
    i++;
  }
  return value;
}

void freeResources ()
// Frees some remaining objects in memory
{
  free (_CGI_formData);
}
